@@ -1,11 +1,13 @@
+import pathlib
 from hub.core.sample import Sample  # type: ignore
-from typing import Optional, Dict
+from typing import Optional, Dict, Union
 
 from hub.core.storage.provider import StorageProvider
+from hub.util.path import convert_pathlib_to_string_if_needed
 
 
 def read(
-    path: str,
+    path: Union[str, pathlib.Path],
     verify: bool = False,
     creds: Optional[Dict] = None,
     compression: Optional[str] = None,
@@ -46,13 +48,14 @@ def read(
     Args:
         path (str): Path to a supported file.
         verify (bool):  If True, contents of the file are verified.
-        creds (optional, Dict): Credentials for s3 and gcp for urls.
+        creds (optional, Dict): Credentials for s3, gcp and http urls.
         compression (optional, str): Format of the file (see `hub.compression.SUPPORTED_COMPRESSIONS`). Only required if path does not have an extension.
         storage (optional, StorageProvider): Storage provider to use to retrieve remote files. Useful if multiple files are being read from same storage to minimize overhead of creating a new provider.
 
     Returns:
         Sample: Sample object. Call `sample.array` to get the `np.ndarray`.
     """
+    path = convert_pathlib_to_string_if_needed(path)
     return Sample(
         path, verify=verify, compression=compression, creds=creds, storage=storage
     )
