@@ -1,4 +1,5 @@
-from hub.util.exceptions import DatasetHandlerError
+from hub.util.exceptions import DatasetHandlerError, UserNotLoggedInException
+from hub.cli.auth import logout
 from click.testing import CliRunner
 import pytest
 import hub
@@ -65,9 +66,18 @@ def test_update_privacy(hub_cloud_ds):
     assert hub_cloud_ds.public
     hub_cloud_ds.make_private()
     assert not hub_cloud_ds.public
-    with pytest.raises(hub.util.exceptions.AuthorizationException):
+
+    runner = CliRunner()
+    runner.invoke(logout)
+    with pytest.raises(UserNotLoggedInException):
+        hub.dataset(hub_cloud_ds.path)
+
+    with pytest.raises(UserNotLoggedInException):
         hub.load(hub_cloud_ds.path)
 
+    with pytest.raises(UserNotLoggedInException):
+        hub.empty(hub_cloud_ds.path)
+
 
 def test_persistence_bug(local_ds_generator):
     for tensor_name in ["abc", "abcd/defg"]:
