@@ -1,6 +1,7 @@
-from unittest.mock import patch
+import hub
 import numpy as np
 import pytest
+from unittest.mock import patch
 
 
 all_non_image_compressions = pytest.mark.parametrize(
@@ -146,3 +147,51 @@ def test_updation_bug(memory_ds):
         np.testing.assert_array_equal(labels[0].numpy(), [0, 1])
         labels[0] = [1, 2]
         np.testing.assert_array_equal(labels[0].numpy(), [1, 2])
+
+
+@patch("hub.constants._ENABLE_RANDOM_ASSIGNMENT", True)
+@pytest.mark.parametrize("insert_first", [False, True])
+def test_insertion_link(memory_ds, insert_first, cat_path, flower_path):
+    with memory_ds as ds:
+        ds.create_tensor("abc", htype="link[image]")
+        first = hub.link(cat_path)
+        tenth = hub.link(flower_path)
+        empty_sample = np.ones((0,))
+        if insert_first:
+            ds.abc.append(first)
+        ds.abc[10] = tenth
+
+        if insert_first:
+            assert ds.abc[0].numpy().shape == (900, 900, 3)
+        else:
+            np.testing.assert_array_equal(ds.abc[0].numpy(), empty_sample)
+
+        for i in range(1, 10):
+            np.testing.assert_array_equal(ds.abc[i].numpy(), empty_sample)
+
+        assert ds.abc[10].numpy().shape == (513, 464, 4)
+
+
+# removing this test until we fix sequence updates
+# @patch("hub.constants._ENABLE_RANDOM_ASSIGNMENT", True)
+# @pytest.mark.parametrize("insert_first", [True])
+# def test_insertion_sequence(memory_ds, insert_first):
+#     with memory_ds as ds:
+#         ds.create_tensor("abc", htype="sequence")
+#         first = [[1, 2, 3, 4], [5, 6, 7, 8]]
+#         tenth = [[9, 10, 11, 12], [13, 14, 15, 16], [17, 18, 19, 20]]
+#         empty_sample = np.ones((0, 0))
+#         if insert_first:
+#             ds.abc.append(first)
+#         ds.abc[10] = tenth
+#         ds.abc[0].numpy()
+
+#         if insert_first:
+#             np.testing.assert_array_equal(ds.abc[0].numpy(), first)
+#         else:
+#             np.testing.assert_array_equal(ds.abc[0].numpy(), empty_sample)
+
+#         for i in range(1, 10):
+#             np.testing.assert_array_equal(ds.abc[i].numpy(), empty_sample)
+
+#         np.testing.assert_array_equal(ds.abc[10].numpy(), tenth)
