@@ -1,6 +1,6 @@
-import ray
-from ray.util.multiprocessing import Pool
-from ray.util.queue import Queue
+import ray  # type: ignore
+from ray.util.multiprocessing import Pool  # type: ignore
+from ray.util.queue import Queue  # type: ignore
 from hub.core.compute.provider import ComputeProvider
 
 
