@@ -1,5 +1,6 @@
 from typing import Any, Dict
 from hub.core.meta.meta import Meta
+from hub.core.index import Index
 import posixpath
 
 
@@ -10,6 +11,7 @@ class DatasetMeta(Meta):
         self.groups = []
         self.tensor_names = {}
         self.hidden_tensors = []
+        self.default_index = Index().to_json()
 
     @property
     def visible_tensors(self):
@@ -31,6 +33,7 @@ class DatasetMeta(Meta):
         d["groups"] = self.groups
         d["tensor_names"] = self.tensor_names
         d["hidden_tensors"] = self.hidden_tensors
+        d["default_index"] = self.default_index
         return d
 
     def add_tensor(self, name, key, hidden=False):
@@ -60,6 +63,13 @@ class DatasetMeta(Meta):
     def delete_group(self, name):
         self.groups = list(filter(lambda g: not g.startswith(name), self.groups))
         self.tensors = list(filter(lambda t: not t.startswith(name), self.tensors))
+        self.hidden_tensors = list(
+            filter(lambda t: not t.startswith(name), self.hidden_tensors)
+        )
+        tensor_names_keys = list(self.tensor_names.keys())
+        for key in tensor_names_keys:
+            if key.startswith(name):
+                self.tensor_names.pop(key)
         self.is_dirty = True
 
     def rename_tensor(self, name, new_name):
@@ -72,7 +82,7 @@ class DatasetMeta(Meta):
         self.groups = list(
             map(
                 lambda g: posixpath.join(new_name, posixpath.relpath(g, name))
-                if g.startswith(name)
+                if (g == name or g.startswith(name + "/"))
                 else g,
                 self.groups,
             )
