@@ -20,11 +20,7 @@ from hub.constants import (
 from hub.compression import (
     COMPRESSION_ALIASES,
 )
-from hub.htype import (
-    HTYPE_CONFIGURATIONS,
-    DEFAULT_HTYPE,
-    HTYPE_SUPPORTED_COMPRESSIONS,
-)
+from hub.htype import HTYPE_CONFIGURATIONS, HTYPE_SUPPORTED_COMPRESSIONS, htype as HTYPE
 from hub.htype import HTYPE_CONFIGURATIONS, REQUIRE_USER_SPECIFICATION, UNSPECIFIED
 from hub.core.meta.meta import Meta
 from hub.core.tensor_link import get_link_transform
@@ -40,6 +36,7 @@ class TensorMeta(Meta):
     sample_compression: str
     chunk_compression: str
     max_chunk_size: int
+    tiling_threshold: int
     hidden: bool
     links: Dict[str, Dict[str, Union[str, bool]]]
     is_sequence: bool
@@ -63,10 +60,10 @@ class TensorMeta(Meta):
         """
 
         super().__init__()
-        if htype and htype not in [UNSPECIFIED, DEFAULT_HTYPE]:
+        if htype and htype not in [UNSPECIFIED, HTYPE.DEFAULT]:
             self.set_htype(htype, **kwargs)
         else:
-            self.set_htype(DEFAULT_HTYPE, **kwargs)
+            self.set_htype(HTYPE.DEFAULT, **kwargs)
             self.htype = None  # type: ignore
 
     def add_link(
@@ -78,8 +75,9 @@ class TensorMeta(Meta):
         }
         if update_f is not None:
             link["update"] = update_f
-        _validate_links({"name": link})
-        self.links[name] = link  # type: ignore
+        d = {name: link}
+        _validate_links(d)
+        self.links.update(d)  # type: ignore
         self.is_dirty = True
 
     def set_hidden(self, val: bool):
@@ -94,9 +92,6 @@ class TensorMeta(Meta):
                 f"Tensor meta already has a dtype ({self.dtype}). Incoming: {dtype.name}."
             )
 
-        if self.length > 0:
-            raise ValueError("Dtype was None, but length was > 0.")
-
         self.dtype = dtype.name
         self.is_dirty = True
 
@@ -112,9 +107,6 @@ class TensorMeta(Meta):
                 f"Tensor meta already has a htype ({self.htype}). Incoming: {htype}."
             )
 
-        if getattr(self, "length", 0) > 0:
-            raise ValueError("Htype was None, but length was > 0.")
-
         if not kwargs:
             kwargs = HTYPE_CONFIGURATIONS[htype]
 
@@ -164,7 +156,7 @@ class TensorMeta(Meta):
         if length != 0:
             self.is_dirty = True
 
-    def _pop(self):
+    def pop(self, index):
         self.length -= 1
         if self.length == 0:
             self.min_shape = []
@@ -265,12 +257,12 @@ def _validate_htype_overwrites(htype: str, htype_overwrite: dict):
     sc = htype_overwrite["sample_compression"]
     cc = htype_overwrite["chunk_compression"]
     compr = sc if cc in (None, UNSPECIFIED) else cc
-    if htype == "image" and sc == UNSPECIFIED and cc == UNSPECIFIED:
+    if htype.startswith("image") and sc == UNSPECIFIED and cc == UNSPECIFIED:
         if not htype_overwrite["is_link"]:
             raise TensorMetaMissingRequiredValue(
                 htype, ["chunk_compression", "sample_compression"]  # type: ignore
             )
-    if htype in ("audio", "video"):
+    if htype in ("audio", "video", "point_cloud"):
         if cc not in (UNSPECIFIED, None):
             raise UnsupportedCompressionError("Chunk compression", htype=htype)
         elif sc == UNSPECIFIED:
@@ -297,7 +289,10 @@ def _replace_unspecified_values(htype: str, htype_overwrite: dict):
         if v == UNSPECIFIED:
             htype_overwrite[k] = defaults[k]
 
-    if htype in ("json", "list", "text") and not htype_overwrite["dtype"]:
+    if (
+        htype in ("json", "list", "text", "point_cloud_calibration_matrix")
+        and not htype_overwrite["dtype"]
+    ):
         htype_overwrite["dtype"] = HTYPE_CONFIGURATIONS[htype]["dtype"]
 
 
@@ -343,7 +338,7 @@ def _format_values(htype: str, htype_overwrite: dict):
 
     dtype = htype_overwrite["dtype"]
     if dtype is not None:
-        if htype in ("json", "list"):
+        if htype in ("json", "list", "point_cloud_calibration_matrix"):
             if getattr(dtype, "__module__", None) == "typing":
                 htype_overwrite["dtype"] = str(dtype)
         else:
