@@ -1,4 +1,4 @@
-from typing import Any, Callable, List, Union
+from typing import Any, Callable, List, Union, Optional
 from hub.core.dataset import Dataset
 from hub.core.io import IOBlock, SampleStreaming
 from hub.core.index import Index
@@ -37,7 +37,6 @@ class DatasetQuery:
 
     def execute(self) -> List[int]:
         idx_map: List[int] = list()
-        idx: int = 0
 
         for f, blk in zip(self._np_access, self._blocks):
             cache = {tensor: f(tensor) for tensor in self._tensors}
@@ -48,7 +47,8 @@ class DatasetQuery:
                 }
                 p.update(self._groups)
                 if eval(self._cquery, p):
-                    idx_map.append(local_idx)
+                    global_index = blk.indices()[local_idx]
+                    idx_map.append(global_index)
                     self._pg_callback(local_idx, True)
                 else:
                     self._pg_callback(local_idx, False)
@@ -90,7 +90,7 @@ def _get_np(dataset: Dataset, block: IOBlock) -> NP_ACCESS:
     def f(tensor: str) -> NP_RESULT:
         tensor_obj = dataset.tensors[tensor]
         tensor_obj.index = Index()
-        return tensor_obj[idx].numpy(aslist=tensor_obj.is_dynamic)
+        return tensor_obj[idx]
 
     return f
 
@@ -108,98 +108,129 @@ def export_tensor(tensor: Tensor):
 
 class EvalObject:
     def __init__(self) -> None:
-        super().__init__()
         self._val: Any = None
+        self._numpy: Optional[Union[np.ndarray, List[np.ndarray]]] = None
 
     @property
-    def val(self):
+    def value(self):
         return self._val
 
     def with_value(self, v: Any):
         self._val = v
+        self._numpy = None
         return self
 
+    @property
+    def numpy_value(self):
+        if self._numpy is None:
+            self._numpy = self._val.numpy(
+                aslist=self._val.is_dynamic, fetch_chunks=True
+            )
+        return self._numpy
+
     def contains(self, v: Any):
-        return v in self.val
+        return v in self.numpy_value
 
     def __getitem__(self, item):
         r = EvalObject()
-        return r.with_value(self._val[item])
+        return r.with_value(self.value[item])
 
     @property
     def min(self):
         """Returns np.min() for the tensor"""
-        return np.amin(self.val)
+        return np.amin(self.numpy_value)
 
     @property
     def max(self):
         """Returns np.max() for the tensor"""
-        return np.amax(self.val)
+        return np.amax(self.numpy_value)
 
     @property
     def mean(self):
         """Returns np.mean() for the tensor"""
-        return self.val.mean()
+        return self.numpy_value.mean()
 
     @property
     def shape(self):
         """Returns shape of the underlying numpy array"""
-        return self.val.shape  # type: ignore
+        return self.value.shape  # type: ignore
 
     @property
     def size(self):
         """Returns size of the underlying numpy array"""
-        return self.val.size  # type: ignore
+        return self.value.size  # type: ignore
+
+    def _to_np(self, o):
+        if isinstance(o, EvalObject):
+            return o.numpy_value
+        return o
 
     def __eq__(self, o: object) -> bool:
-        if isinstance(self.val, (list, np.ndarray)):
+        val = self.numpy_value
+        o = self._to_np(o)
+        if isinstance(val, (list, np.ndarray)):
             if isinstance(o, (list, tuple)):
-                return set(o) == set(self.val)
+                return set(o) == set(val)
             else:
-                return o in self.val
+                return o in val
         else:
-            return self.val == o
+            return val == o
 
     def __lt__(self, o: object) -> bool:
-        return self.val < o
+        o = self._to_np(o)
+        return self.numpy_value < o
 
     def __le__(self, o: object) -> bool:
-        return self.val <= o
+        o = self._to_np(o)
+        return self.numpy_value <= o
 
     def __gt__(self, o: object) -> bool:
-        return self.val > o
+        o = self._to_np(o)
+        return self.numpy_value > o
 
     def __ge__(self, o: object) -> bool:
-        return self.val >= o
+        o = self._to_np(o)
+        return self.numpy_value >= o
 
     def __mod__(self, o: object):
-        return self.val % o
+        o = self._to_np(o)
+        return self.numpy_value % o
 
     def __add__(self, o: object):
-        return self.val + o
+        o = self._to_np(o)
+        return self.numpy_value + o
 
     def __sub__(self, o: object):
-        return self.val - o
+        o = self._to_np(o)
+        return self.numpy_value - o
 
     def __div__(self, o: object):
-        return self.val / o
+        o = self._to_np(o)
+        return self.numpy_value / o
 
     def __floordiv__(self, o: object):
-        return self.val // o
+        o = self._to_np(o)
+        return self.numpy_value // o
 
     def __mul__(self, o: object):
-        return self.val * o
+        o = self._to_np(o)
+        return self.numpy_value * o
 
     def __pow__(self, o: object):
-        return self.val**o
+        o = self._to_np(o)
+        return self.numpy_value**o
 
     def __contains__(self, o: object):
+        o = self._to_np(o)
         return self.contains(o)
 
+    @property
+    def sample_info(self):
+        return self._val.sample_info
+
 
 class GroupTensor:
     def __init__(self, dataset: Dataset, wrappers, prefix: str) -> None:
-        super().__init__()
         self.prefix = prefix
         self.dataset = dataset
         self.wrappers = wrappers
@@ -229,13 +260,12 @@ class GroupTensor:
 
 class ClassLabelsTensor(EvalObject):
     def __init__(self, tensor: Tensor) -> None:
-        super().__init__()
-        self._tensor = tensor
-
+        super(ClassLabelsTensor, self).__init__()
         _classes = tensor.info["class_names"]  # type: ignore
         self._classes_dict = {v: idx for idx, v in enumerate(_classes)}
 
     def _norm_labels(self, o: object):
+        o = self._to_np(o)
         if isinstance(o, str):
             return self._classes_dict[o]
         elif isinstance(o, int):
@@ -251,26 +281,31 @@ class ClassLabelsTensor(EvalObject):
         return super(ClassLabelsTensor, self).__eq__(o)
 
     def __lt__(self, o: object) -> bool:
+        o = self._to_np(o)
         if isinstance(o, str):
             raise ValueError("label class is not comparable")
-        return self.val < o
+        return self.numpy_value < o
 
     def __le__(self, o: object) -> bool:
+        o = self._to_np(o)
         if isinstance(o, str):
             raise ValueError("label class is not comparable")
-        return self.val <= o
+        return self.numpy_value <= o
 
     def __gt__(self, o: object) -> bool:
+        o = self._to_np(o)
         if isinstance(o, str):
             raise ValueError("label class is not comparable")
-        return self.val > o
+        return self.numpy_value > o
 
     def __ge__(self, o: object) -> bool:
+        o = self._to_np(o)
         if isinstance(o, str):
             raise ValueError("label class is not comparable")
-        return self.val >= o
+        return self.numpy_value >= o
 
     def contains(self, v: Any):
+        v = self._to_np(v)
         if isinstance(v, str):
             v = self._classes_dict[v]
         return super(ClassLabelsTensor, self).contains(v)
