@@ -1,3 +1,4 @@
+import requests
 from hub.core.compression import (
     compress_array,
     decompress_array,
@@ -7,30 +8,26 @@ from hub.core.compression import (
     _open_video,
     _read_metadata_from_vstream,
     _read_audio_meta,
+    _read_point_cloud_meta,
 )
 from hub.compression import (
     get_compression_type,
     AUDIO_COMPRESSION,
     IMAGE_COMPRESSION,
     VIDEO_COMPRESSION,
+    POINT_CLOUD_COMPRESSION,
 )
-from hub.compression import (
-    get_compression_type,
-    AUDIO_COMPRESSION,
-    IMAGE_COMPRESSION,
-)
+from hub.util.exceptions import UnableToReadFromUrlError
 from hub.util.exif import getexif
 from hub.core.storage.provider import StorageProvider
 from hub.util.path import get_path_type, is_remote_path
 import numpy as np
-from typing import List, Optional, Tuple, Union, Dict
+from typing import Optional, Tuple, Union, Dict
 
 from PIL import Image  # type: ignore
 from PIL.ExifTags import TAGS  # type: ignore
 from io import BytesIO
 
-from urllib.request import urlopen
-
 from hub.core.storage.s3 import S3Provider
 from hub.core.storage.google_drive import GDriveProvider
 
@@ -71,8 +68,8 @@ class Sample:
             compression (str): Specify in case of byte buffer.
             verify (bool): If a path is provided, verifies the sample if True.
             shape (Tuple[int]): Shape of the sample.
-            dtype (str, optional): Data type of the sample.
-            creds (optional, Dict): Credentials for s3 and gcp for urls.
+            dtype (optional, str): Data type of the sample.
+            creds (optional, Dict): Credentials for s3, gcp and http urls.
             storage (optional, StorageProvider): Storage provider.
 
         Raises:
@@ -88,21 +85,29 @@ class Sample:
         self._typestr = None
         self._shape = shape or None
         self._dtype = dtype or None
+
         self.path = None
         self.storage = storage
         self._buffer = None
         self._creds = creds or {}
+        self._verify = verify
 
         if path is not None:
             self.path = path
             self._compression = compression
-            self._verified = False
-            self._verify = verify
+            if self._verify:
+                if self._compression is None:
+                    self._compression = get_compression(path=self.path)
+                compressed_bytes = self._read_from_path()
+                if self._compression is None:
+                    self._compression = get_compression(header=compressed_bytes[:32])
+                self._shape, self._typestr = verify_compressed_file(compressed_bytes, self._compression)  # type: ignore
 
         if array is not None:
             self._array = array
             self._shape = array.shape  # type: ignore
             self._typestr = array.__array_interface__["typestr"]
+            self._dtype = np.dtype(self._typestr).name
             self._compression = None
 
         if buffer is not None:
@@ -112,13 +117,23 @@ class Sample:
                 self._uncompressed_bytes = buffer
             else:
                 self._compressed_bytes[compression] = buffer
+                if self._verify:
+                    self._shape, self._typestr = verify_compressed_file(buffer, self._compression)  # type: ignore
+
+        self.htype = None
 
     @property
     def buffer(self):
+        if self._buffer is None and self.path is not None:
+            self._read_from_path()
         if self._buffer is not None:
             return self._buffer
         return self.compressed_bytes(self.compression)
 
+    @property
+    def is_text_like(self):
+        return self.htype in {"text", "list", "json"}
+
     @property
     def dtype(self):
         if self._dtype is None:
@@ -218,6 +233,13 @@ class Sample:
             info = _read_audio_meta(self.buffer)
         return info
 
+    def _get_point_cloud_meta(self) -> dict:
+        if self.path and get_path_type(self.path) == "local":
+            info = _read_point_cloud_meta(self.path)
+        else:
+            info = _read_point_cloud_meta(self.buffer)
+        return info
+
     @property
     def is_lazy(self) -> bool:
         return self._array is None
@@ -238,7 +260,7 @@ class Sample:
             self._uncompressed_bytes = img.tobytes()
         return compress_array(self.array, compression)
 
-    def compressed_bytes(self, compression: str) -> bytes:
+    def compressed_bytes(self, compression: Optional[str]) -> bytes:
         """Returns this sample as compressed bytes.
 
         Note:
@@ -246,7 +268,7 @@ class Sample:
                 returned without re-compressing.
 
         Args:
-            compression (str): `self.array` will be compressed into this format. If `compression is None`, return `self.uncompressed_bytes()`.
+            compression (optional, str): `self.array` will be compressed into this format. If `compression is None`, return `self.uncompressed_bytes()`.
 
         Returns:
             bytes: Bytes for the compressed sample. Contains all metadata required to decompress within these bytes.
@@ -256,7 +278,7 @@ class Sample:
         """
 
         if compression is None:
-            return self.uncompressed_bytes()
+            return self.uncompressed_bytes()  # type: ignore
 
         compressed_bytes = self._compressed_bytes.get(compression)
         if compressed_bytes is None:
@@ -267,11 +289,7 @@ class Sample:
                 if self._compression is None:
                     self._compression = get_compression(header=compressed_bytes[:32])
                 if self._compression == compression:
-                    if self._verify:
-                        self._shape, self._typestr = verify_compressed_file(  # type: ignore
-                            compressed_bytes, self._compression
-                        )
-                    elif self._shape is None:
+                    if self._shape is None:
                         _, self._shape, self._typestr = read_meta_from_compressed_file(
                             compressed_bytes, compression=self._compression
                         )
@@ -289,28 +307,43 @@ class Sample:
             if self._uncompressed_bytes is None:
                 self._uncompressed_bytes = self._array.tobytes()
             return
-        if self.path and get_path_type(self.path) == "local":
-            compressed = self.path
-        else:
-            compressed = self.buffer
-        self._array = decompress_array(
-            compressed, compression=self.compression, shape=self.shape, dtype=self.dtype
-        )
-        self._uncompressed_bytes = self._array.tobytes()
-        self._typestr = self._array.__array_interface__["typestr"]
-        self._dtype = np.dtype(self._typestr).name
+        compression = self.compression
+        if compression is None and self._buffer is not None:
+            if self.is_text_like:
+                from hub.core.serialize import bytes_to_text
 
-    def uncompressed_bytes(self) -> bytes:
+                buffer = bytes(self._buffer)
+                self._array = bytes_to_text(buffer, self.htype)
+            else:
+                self._array = np.frombuffer(self._buffer, dtype=self.dtype).reshape(
+                    self.shape
+                )
+
+        else:
+            if self.path and get_path_type(self.path) == "local":
+                compressed = self.path
+            else:
+                compressed = self.buffer
+
+            self._array = decompress_array(
+                compressed, compression=compression, shape=self.shape, dtype=self.dtype
+            )
+            self._uncompressed_bytes = self._array.tobytes()
+            self._typestr = self._array.__array_interface__["typestr"]
+            self._dtype = np.dtype(self._typestr).name
+
+    def uncompressed_bytes(self) -> Optional[bytes]:
         """Returns uncompressed bytes."""
         self._decompress()
-        assert self._uncompressed_bytes is not None
         return self._uncompressed_bytes
 
     @property
-    def array(self) -> np.ndarray:
+    def array(self) -> np.ndarray:  # type: ignore
+        arr = self._array
+        if arr is not None:
+            return arr
         self._decompress()
-        assert self._array is not None
-        return self._array
+        return self._array  # type: ignore
 
     def __str__(self):
         if self.is_lazy:
@@ -333,17 +366,19 @@ class Sample:
         return self.buffer == other.buffer
 
     def _read_from_path(self) -> bytes:  # type: ignore
-        path_type = get_path_type(self.path)
-        if path_type == "local":
-            return self._read_from_local()
-        elif path_type == "gcs":
-            return self._read_from_gcs()
-        elif path_type == "s3":
-            return self._read_from_s3()
-        elif path_type == "gdrive":
-            return self._read_from_gdrive()
-        elif path_type == "http":
-            return self._read_from_http()
+        if self._buffer is None:
+            path_type = get_path_type(self.path)
+            if path_type == "local":
+                self._buffer = self._read_from_local()
+            elif path_type == "gcs":
+                self._buffer = self._read_from_gcs()
+            elif path_type == "s3":
+                self._buffer = self._read_from_s3()
+            elif path_type == "gdrive":
+                self._buffer = self._read_from_gdrive()
+            elif path_type == "http":
+                self._buffer = self._read_from_http()
+        return self._buffer  # type: ignore
 
     def _read_from_local(self) -> bytes:
         with open(self.path, "rb") as f:  # type: ignore
@@ -378,17 +413,24 @@ class Sample:
             return self.storage.get_object_from_full_url(self.path)
         path = self.path.replace("gcp://", "").replace("gcs://", "")  # type: ignore
         root, key = self._get_root_and_key(path)
-        gcs = GCSProvider(root, **self._creds)
+        gcs = GCSProvider(root, token=self._creds)
         return gcs[key]
 
     def _read_from_gdrive(self) -> bytes:
-        path = self.path.replace("gdrive://", "")  # type: ignore
-        root, key = self._get_root_and_key(path)
-        gdrive = GDriveProvider("gdrive://" + root, token=self._creds)
-        return gdrive[key]
+        assert self.path is not None
+        gdrive = GDriveProvider("gdrive://", token=self._creds, makemap=False)
+        return gdrive.get_object_from_full_url(self.path)
 
     def _read_from_http(self) -> bytes:
-        return urlopen(self.path).read()  # type: ignore
+        assert self.path is not None
+        if "Authorization" in self._creds:
+            headers = {"Authorization": self._creds["Authorization"]}
+        else:
+            headers = {}
+        result = requests.get(self.path, headers=headers)
+        if result.status_code != 200:
+            raise UnableToReadFromUrlError(self.path, result.status_code)
+        return result.content
 
     def _getexif(self) -> dict:
         if self.path and get_path_type(self.path) == "local":
@@ -416,6 +458,8 @@ class Sample:
             meta.update(self._get_video_meta())
         elif compression_type == AUDIO_COMPRESSION:
             meta.update(self._get_audio_meta())
+        elif compression_type == POINT_CLOUD_COMPRESSION:
+            meta.update(self._get_point_cloud_meta())
         meta["shape"] = self.shape
         meta["format"] = self.compression
         if self.path:
