@@ -5,7 +5,11 @@ import shutil
 from typing import Optional, Set
 
 from hub.core.storage.provider import StorageProvider
-from hub.util.exceptions import DirectoryAtPathException, FileAtPathException
+from hub.util.exceptions import (
+    DirectoryAtPathException,
+    FileAtPathException,
+    PathNotEmptyException,
+)
 
 
 class LocalProvider(StorageProvider):
@@ -180,14 +184,57 @@ class LocalProvider(StorageProvider):
             raise DirectoryAtPathException
         return full_path
 
-    def clear(self):
-        """Deletes ALL data on the local machine (under self.root). Exercise caution!"""
+    def clear(self, prefix=""):
+        """Deletes ALL data with keys having given prefix on the local machine (under self.root). Exercise caution!"""
         self.check_readonly()
-        self.files = set()
         full_path = os.path.expanduser(self.root)
+        if prefix and self.files:
+            self.files = set(file for file in self.files if not file.startswith(prefix))
+            full_path = os.path.join(full_path, prefix)
+        else:
+            self.files = set()
         if os.path.exists(full_path):
             shutil.rmtree(full_path)
 
+    def rename(self, path):
+        """Renames root folder"""
+        if os.path.isfile(path) or (os.path.isdir(path) and len(os.listdir(path)) > 0):
+            raise PathNotEmptyException(use_hub=False)
+        os.rename(self.root, path)
+        self.root = path
+
     def __contains__(self, key) -> bool:
         full_path = self._check_is_file(key)
         return os.path.exists(full_path)
+
+    def __getstate__(self):
+        return self.root
+
+    def __setstate__(self, state):
+        self.__init__(state)
+
+    def get_presigned_url(self, key: str) -> str:
+        return os.path.join(self.root, key)
+
+    def get_object_size(self, key: str) -> int:
+        return os.stat(os.path.join(self.root, key)).st_size
+
+    def get_bytes(
+        self,
+        path: str,
+        start_byte: Optional[int] = None,
+        end_byte: Optional[int] = None,
+    ):
+        try:
+            full_path = self._check_is_file(path)
+            with open(full_path, "rb") as file:
+                if start_byte is not None:
+                    file.seek(start_byte)
+                if end_byte is None:
+                    return file.read()
+                else:
+                    return file.read(end_byte - (start_byte or 0))
+        except DirectoryAtPathException:
+            raise
+        except FileNotFoundError:
+            raise KeyError(path)
