@@ -12,6 +12,7 @@ from hub.core.index import Index, IndexEntry
 from hub.core.meta.tensor_meta import TensorMeta
 from hub.core.storage import StorageProvider
 from hub.core.chunk_engine import ChunkEngine
+from hub.core.compression import _read_timestamps
 from hub.core.tensor_link import get_link_transform
 from hub.api.info import load_info
 from hub.util.keys import (
@@ -35,6 +36,7 @@ from hub.util.keys import (
     get_sample_shape_tensor_key,
 )
 from hub.util.modified import get_modified_indexes
+from hub.util.class_label import convert_to_text
 from hub.util.shape_interval import ShapeInterval
 from hub.util.exceptions import (
     TensorDoesNotExistError,
@@ -47,13 +49,18 @@ from hub.util.pretty_print import (
     get_string,
     summary_tensor,
 )
-from hub.constants import FIRST_COMMIT_ID, MB, _NO_LINK_UPDATE
+from hub.constants import FIRST_COMMIT_ID, _NO_LINK_UPDATE, UNSPECIFIED
 
 
 from hub.util.version_control import auto_checkout
+from hub.util.video import normalize_index
 
 from hub.compression import get_compression_type, VIDEO_COMPRESSION
 from hub.util.notebook import is_jupyter, video_html, is_colab
+from hub.util.point_cloud import (
+    POINT_CLOUD_FIELD_NAME_TO_TYPESTR,
+    cast_point_cloud_array_to_proper_dtype,
+)
 import warnings
 import webbrowser
 
@@ -243,12 +250,16 @@ class Tensor:
         else:
             self.chunk_engine = ChunkEngine(self.key, self.storage, self.version_state)
 
-        if not self.is_iteration:
+        if not self.pad_tensor and not self.is_iteration:
             self.index.validate(self.num_samples)
 
         # An optimization to skip multiple .numpy() calls when performing inplace ops on slices:
         self._skip_next_setitem = False
 
+    @property
+    def pad_tensor(self):
+        return self.dataset._pad_tensors
+
     def _write_initialization(self):
         self.storage.check_readonly()
         # if not the head node, checkout to an auto branch that is newly created
@@ -258,7 +269,11 @@ class Tensor:
             ].chunk_engine
 
     @invalid_view_op
-    def extend(self, samples: Union[np.ndarray, Sequence[InputSample], "Tensor"]):
+    def extend(
+        self,
+        samples: Union[np.ndarray, Sequence[InputSample], "Tensor"],
+        progressbar: bool = False,
+    ):
 
         """Extends the end of the tensor by appending multiple elements from a sequence. Accepts a sequence, a single batched numpy array,
         or a sequence of `hub.read` outputs, which can be used to load files. See examples down below.
@@ -288,14 +303,17 @@ class Tensor:
         Args:
             samples (np.ndarray, Sequence, Sequence[Sample]): The data to add to the tensor.
                 The length should be equal to the number of samples to add.
+            progressbar (bool): Specifies whether a progressbar should be displayed while extending.
 
         Raises:
             TensorDtypeMismatchError: TensorDtypeMismatchError: Dtype for array must be equal to or castable to this tensor's dtype
         """
-        self.check_link_ready()
         self._write_initialization()
+        [f() for f in list(self.dataset._update_hooks.values())]
         self.chunk_engine.extend(
-            samples, link_callback=self._append_to_links if self.meta.links else None
+            samples,
+            progressbar=progressbar,
+            link_callback=self._append_to_links if self.meta.links else None,
         )
 
     @property
@@ -323,10 +341,7 @@ class Tensor:
             raise TypeError("Info must be set with type Dict")
 
     @invalid_view_op
-    def append(
-        self,
-        sample: InputSample,
-    ):
+    def append(self, sample: InputSample):
         """Appends a single sample to the end of the tensor. Can be an array, scalar value, or the return value from `hub.read`,
         which can be used to load files. See examples down below.
 
@@ -350,7 +365,7 @@ class Tensor:
         Args:
             sample (InputSample): The data to append to the tensor. `Sample` is generated by `hub.read`. See the above examples.
         """
-        self.extend([sample])
+        self.extend([sample], progressbar=False)
 
     def clear(self):
         """Deletes all samples from the tensor"""
@@ -422,11 +437,24 @@ class Tensor:
             if sample_shape_tensor
             else None
         )
-        if sample_shape_provider is None:
-            self.check_link_ready()
-        return self.chunk_engine.shape(
+        shape: Tuple[Optional[int], ...]
+        shape = self.chunk_engine.shape(
             self.index, sample_shape_provider=sample_shape_provider
         )
+        if not shape and self.meta.max_shape:
+            shape = (0,) * len(self.meta.max_shape)
+        if self.meta.max_shape == [0, 0, 0]:
+            shape = ()
+        return shape
+
+    @property
+    def size(self) -> Optional[int]:
+        s = 1
+        for x in self.shape:
+            if x is None:
+                return None
+            s *= x  # not using np.prod to avoid overflow
+        return s
 
     @property
     def ndim(self) -> int:
@@ -501,7 +529,7 @@ class Tensor:
         """
         if self.is_sequence:
             return self.chunk_engine._sequence_length
-        return self.meta.length
+        return self.chunk_engine.num_samples
 
     def __len__(self):
         """Returns the length of the primary axis of the tensor.
@@ -580,8 +608,8 @@ class Tensor:
             >>> tensor.shape
             (1, 3, 3)
         """
-        self.check_link_ready()
         self._write_initialization()
+        [f() for f in list(self.dataset._update_hooks.values())]
         update_link_callback = self._update_links if self.meta.links else None
         if isinstance(value, Tensor):
             if value._skip_next_setitem:
@@ -595,6 +623,10 @@ class Tensor:
             and isinstance(item, int)
             and item >= self.num_samples
         ):
+            if self.is_sequence:
+                raise NotImplementedError(
+                    "Random assignment is not supported for sequences yet."
+                )
             num_samples_to_pad = item - self.num_samples
             append_link_callback = self._append_to_links if self.meta.links else None
 
@@ -606,6 +638,10 @@ class Tensor:
             )
             return
 
+        if not item_index.values[0].subscriptable() and not self.is_sequence:
+            # we're modifying a single sample, convert it to a list as chunk engine expects multiple samples
+            value = [value]
+
         self.chunk_engine.update(
             self.index[item_index],
             value,
@@ -616,22 +652,19 @@ class Tensor:
         for i in range(len(self)):
             yield self.__getitem__(i, is_iteration=True)
 
-    def check_link_ready(self):
-        if not self.is_link:
-            return
-        missing_keys = self.link_creds.missing_keys
-        if missing_keys:
-            raise ValueError(
-                f"Not all credentials are populated for a tensor with linked data. Missing: {missing_keys}. Populate with `dataset.populate_creds(key, value)`."
-            )
-
-    def numpy(self, aslist=False) -> Union[np.ndarray, List[np.ndarray]]:
+    def numpy(
+        self, aslist=False, fetch_chunks=False
+    ) -> Union[np.ndarray, List[np.ndarray]]:
         """Computes the contents of the tensor in numpy format.
 
         Args:
             aslist (bool): If True, a list of np.ndarrays will be returned. Helpful for dynamic tensors.
                 If False, a single np.ndarray will be returned unless the samples are dynamically shaped, in which case
                 an error is raised.
+            fetch_chunks (bool): If True, full chunks will be retrieved from the storage, otherwise only required bytes will be retrieved.
+                This will always be True even if specified as False in the following cases:
+                - The tensor is ChunkCompressed
+                - The chunk which is being accessed has more than 128 samples.
 
         Raises:
             DynamicTensorNumpyError: If reading a dynamically-shaped array slice without `aslist=True`.
@@ -640,8 +673,24 @@ class Tensor:
         Returns:
             A numpy array containing the data represented by this tensor.
         """
-        self.check_link_ready()
-        return self.chunk_engine.numpy(self.index, aslist=aslist)
+        if self.htype == "point_cloud":
+            full_numpy_arr = self.chunk_engine.numpy(
+                self.index,
+                aslist=aslist,
+                fetch_chunks=fetch_chunks,
+                pad_tensor=self.pad_tensor,
+            )
+
+            if isinstance(full_numpy_arr, list):
+                return [arr[..., :3] for arr in full_numpy_arr]
+            return full_numpy_arr[..., :3]
+
+        return self.chunk_engine.numpy(
+            self.index,
+            aslist=aslist,
+            fetch_chunks=fetch_chunks,
+            pad_tensor=self.pad_tensor,
+        )
 
     def summary(self):
         pretty_print = summary_tensor(self)
@@ -708,21 +757,101 @@ class Tensor:
     def __ior__(self, other):
         pass
 
-    def data(self) -> Any:
+    def data(self, aslist: bool = False) -> Any:
         htype = self.base_htype
         if htype in ("json", "text"):
 
             if self.ndim == 1:
-                return self.numpy()[0]
+                return {"value": self.numpy()[0]}
             else:
-                return [sample[0] for sample in self.numpy(aslist=True)]
+                return {"value": [sample[0] for sample in self.numpy(aslist=True)]}
         elif htype == "list":
             if self.ndim == 1:
-                return list(self.numpy())
+                return {"value": list(self.numpy())}
             else:
-                return list(map(list, self.numpy(aslist=True)))
+                return {"value": list(map(list, self.numpy(aslist=True)))}
+        elif self.htype == "video":
+            data = {}
+            data["frames"] = self.numpy(aslist=aslist)
+            index = self.index
+            if index.values[0].subscriptable():
+                root = Tensor(self.key, self.dataset)
+                if len(index.values) > 1:
+                    data["timestamps"] = np.array(
+                        [
+                            root[i, index.values[1].value].timestamps  # type: ignore
+                            for i in index.values[0].indices(self.num_samples)
+                        ]
+                    )
+                else:
+                    data["timestamps"] = np.array(
+                        [
+                            root[i].timestamps
+                            for i in index.values[0].indices(self.num_samples)
+                        ]
+                    )
+            else:
+                data["timestamps"] = self.timestamps
+            if aslist:
+                data["timestamps"] = data["timestamps"].tolist()  # type: ignore
+
+            data["sample_info"] = self.sample_info
+            return data
+        elif htype == "class_label":
+            labels = self.numpy(aslist=aslist)
+            data = {"value": labels}
+            class_names = self.info.class_names
+            if class_names:
+                data["text"] = convert_to_text(labels, self.info.class_names)
+            return data
+        elif htype in ("image", "image.rgb", "image.gray", "dicom"):
+            return {
+                "value": self.numpy(aslist=aslist),
+                "sample_info": self.sample_info or {},
+            }
+        elif htype == "point_cloud":
+            full_arr = self.chunk_engine.numpy(
+                self.index,
+                aslist=aslist,
+                pad_tensor=self.pad_tensor,
+            )
+
+            if self.ndim == 2:
+                meta = {}  # type: ignore
+
+                if len(self.sample_info) == 0:
+                    return meta
+
+                for i, dimension_name in enumerate(self.sample_info["dimension_names"]):
+                    typestr = POINT_CLOUD_FIELD_NAME_TO_TYPESTR[dimension_name]
+                    meta[dimension_name] = full_arr[..., i].astype(np.dtype(typestr))  # type: ignore
+                return meta
+
+            meta = []  # type: ignore
+            for sample_index in range(len(full_arr)):
+                meta_dict = {}  # type: ignore
+
+                if len(self.sample_info[sample_index]) == 0:
+                    meta.append(meta_dict)  # type: ignore
+                    continue
+
+                for dimension_index, dimension_name in enumerate(
+                    self.sample_info[sample_index]["dimension_names"]
+                ):
+                    dtype = POINT_CLOUD_FIELD_NAME_TO_TYPESTR[dimension_name]
+                    meta_dict[dimension_name] = cast_point_cloud_array_to_proper_dtype(
+                        full_arr, sample_index, dimension_index, dtype
+                    )
+                meta.append(meta_dict)  # type: ignore
+
+            if len(full_arr) == 1:
+                meta = meta[0]  # type: ignore
+            return meta
+
         else:
-            return self.numpy()
+            return {
+                "value": self.numpy(aslist=aslist),
+            }
 
     def tobytes(self) -> bytes:
         """Returns the bytes of the tensor.
@@ -740,22 +869,21 @@ class Tensor:
         """
         if self.index.values[0].subscriptable() or len(self.index.values) > 1:
             raise ValueError("tobytes() can be used only on exatcly 1 sample.")
-        self.check_link_ready()
         idx = self.index.values[0].value
-        if self.is_link:
-            return self.chunk_engine.get_hub_read_sample(idx).compressed_bytes  # type: ignore
         return self.chunk_engine.read_bytes_for_sample(idx)  # type: ignore
 
-    def _pop(self):
-        self.chunk_engine._pop()
-        [self.dataset[link]._pop() for link in self.meta.links]
-
     def _append_to_links(self, sample, flat: Optional[bool]):
         for k, v in self.meta.links.items():
             if flat is None or v["flatten_sequence"] == flat:
-                Tensor(k, self.dataset).append(
-                    get_link_transform(v["append"])(sample, self.link_creds)
-                )
+                v = get_link_transform(v["append"])(sample, self.link_creds)
+                tensor = Tensor(k, self.dataset)
+                if (
+                    isinstance(v, np.ndarray)
+                    and tensor.dtype
+                    and v.dtype != tensor.dtype
+                ):
+                    v = v.astype(tensor.dtype)  # bc
+                tensor.append(v)
 
     def _update_links(
         self,
@@ -769,23 +897,36 @@ class Tensor:
                 fname = v.get("update")
                 if fname:
                     func = get_link_transform(fname)
+                    tensor = Tensor(k, self.dataset)
                     val = func(
                         new_sample,
-                        Tensor(k, self.dataset)[global_sample_index],
+                        tensor[global_sample_index],
                         sub_index=sub_index,
                         partial=not sub_index.is_trivial(),
                         link_creds=self.link_creds,
                     )
                     if val is not _NO_LINK_UPDATE:
-                        Tensor(k, self.dataset)[global_sample_index] = val
+                        if (
+                            isinstance(val, np.ndarray)
+                            and tensor.dtype
+                            and val.dtype != tensor.dtype
+                        ):
+                            val = val.astype(tensor.dtype)  # bc
+                        tensor[global_sample_index] = val
 
     @property
     def _sample_info_tensor(self):
-        return self.dataset._tensors().get(get_sample_info_tensor_key(self.key))
+        ds = self.dataset
+        return ds.version_state["full_tensors"].get(
+            ds.version_state["tensor_names"].get(get_sample_info_tensor_key(self.key))
+        )
 
     @property
     def _sample_shape_tensor(self):
-        return self.dataset._tensors().get(get_sample_shape_tensor_key(self.key))
+        ds = self.dataset
+        return ds.version_state["full_tensors"].get(
+            ds.version_state["tensor_names"].get(get_sample_shape_tensor_key(self.key))
+        )
 
     @property
     def _sample_id_tensor(self):
@@ -795,23 +936,12 @@ class Tensor:
         if self.is_sequence:
 
             def get_sample_shape(global_sample_index: int):
-                shapes = sample_shape_tensor.numpy(
-                    Index(
-                        [
-                            IndexEntry(
-                                slice(
-                                    *self.chunk_engine.sequence_encoder[
-                                        global_sample_index
-                                    ]
-                                )
-                            )
-                        ]
-                    )
-                )
-                return (len(shapes),) + tuple(
-                    int(shapes[0, i]) if np.all(shapes[:, i] == shapes[0, i]) else None
-                    for i in range(shapes.shape[1])
+                seq_pos = slice(
+                    *self.chunk_engine.sequence_encoder[global_sample_index]
                 )
+                idx = Index([IndexEntry(seq_pos)])
+                shapes = sample_shape_tensor[idx].numpy()
+                return shapes
 
         else:
 
@@ -826,7 +956,7 @@ class Tensor:
                 sample_info_tensor[i].data()
                 for i in range(*self.chunk_engine.sequence_encoder[global_sample_index])
             ]
-        return sample_info_tensor[global_sample_index].data()
+        return sample_info_tensor[global_sample_index].data()["value"]
 
     def _sample_info(self, index: Index):
         sample_info_tensor = self._sample_info_tensor
@@ -856,12 +986,18 @@ class Tensor:
         return self.chunk_engine.linked_sample(self.index.values[0].value)
 
     def _get_video_stream_url(self):
+        if self.is_link:
+            return self.chunk_engine.get_video_url(self.index.values[0].value)
+
         from hub.visualizer.video_streaming import get_video_stream_url
 
         return get_video_stream_url(self, self.index.values[0].value)
 
     def play(self):
-        if get_compression_type(self.meta.sample_compression) != VIDEO_COMPRESSION:
+        if (
+            get_compression_type(self.meta.sample_compression) != VIDEO_COMPRESSION
+            and self.htype != "link[video]"
+        ):
             raise Exception("Only supported for video tensors.")
         if self.index.values[0].subscriptable():
             raise ValueError("Video streaming requires exactly 1 sample.")
@@ -878,3 +1014,102 @@ class Tensor:
             )
         else:
             webbrowser.open(self._get_video_stream_url())
+
+    @invalid_view_op
+    def pop(self, index: Optional[int] = None):
+        """Removes an element at the given index."""
+        if index is None:
+            index = self.num_samples - 1
+        self.chunk_engine.pop(index)
+        [self.dataset[link].pop(index) for link in self.meta.links]
+
+    @property
+    def timestamps(self) -> np.ndarray:
+        """Returns timestamps (in seconds) for video sample as numpy array.
+
+        ## Examples:
+
+        Return timestamps for all frames of first video sample
+
+        ```
+        >>> ds.video[0].timestamp
+        ```
+
+        Return timestamps for 5th to 10th frame of first video sample
+
+        ```
+        >>> ds.video[0, 5:10].timestamp
+        array([0.2002    , 0.23356667, 0.26693332, 0.33366665, 0.4004    ],
+        dtype=float32)
+        ```
+
+        """
+        if (
+            get_compression_type(self.meta.sample_compression) != VIDEO_COMPRESSION
+            and self.htype != "link[video]"
+        ):
+            raise Exception("Only supported for video tensors.")
+        index = self.index
+        if index.values[0].subscriptable():
+            raise ValueError("Only supported for exactly 1 video sample.")
+        if self.is_sequence:
+            if len(index.values) == 1 or index.values[1].subscriptable():
+                raise ValueError("Only supported for exactly 1 video sample.")
+            sub_index = index.values[2].value if len(index.values) > 2 else None
+        else:
+            sub_index = index.values[1].value if len(index.values) > 1 else None
+        global_sample_index = next(index.values[0].indices(self.num_samples))
+        if self.is_link:
+            sample = self.chunk_engine.get_video_url(global_sample_index)  # type: ignore
+        else:
+            sample = self.chunk_engine.get_video_sample(
+                global_sample_index, index, decompress=False
+            )
+
+        nframes = self.shape[0]
+        start, stop, step, reverse = normalize_index(sub_index, nframes)
+
+        stamps = _read_timestamps(sample, start, stop, step, reverse)
+        return stamps
+
+    @property
+    def _config(self):
+        """Returns a summary of the configuration of the tensor."""
+        tensor_meta = self.meta
+        return {
+            "htype": tensor_meta.htype or UNSPECIFIED,
+            "dtype": tensor_meta.dtype or UNSPECIFIED,
+            "sample_compression": tensor_meta.sample_compression or UNSPECIFIED,
+            "chunk_compression": tensor_meta.chunk_compression or UNSPECIFIED,
+            "hidden": tensor_meta.hidden,
+            "is_link": tensor_meta.is_link,
+            "is_sequence": tensor_meta.is_sequence,
+        }
+
+    @property
+    def sample_indices(self):
+        return self.dataset._sample_indices(self.num_samples)
+
+    def _extract_value(self, htype):
+        if self.base_htype != htype:
+            raise Exception(f"Only supported for {htype} tensors.")
+
+        if self.ndim == 1:
+            return self.numpy()[0]
+        else:
+            return [sample[0] for sample in self.numpy(aslist=True)]
+
+    def text(self):
+        return self._extract_value("text")
+
+    def dict(self):
+        return self._extract_value("json")
+
+    def list(self):
+        if self.base_htype != "list":
+            raise Exception(f"Only supported for list tensors.")
+
+        if self.ndim == 1:
+            return list(self.numpy())
+        else:
+            return list(map(list, self.numpy(aslist=True)))
