@@ -6,6 +6,7 @@ from .common import (
     convert_fn as default_convert_fn,
     collate_fn as default_collate_fn,
 )
+from hub.util.exceptions import EmptyTensorError
 
 
 def create_dataloader_nesteddataloader(
@@ -20,6 +21,8 @@ def create_dataloader_nesteddataloader(
     collate_fn,
     pin_memory,
     drop_last,
+    return_index,
+    pad_tensors,
 ):
     import torch
     import torch.utils.data
@@ -34,8 +37,11 @@ def create_dataloader_nesteddataloader(
             tobytes=tobytes,
             use_local_cache=use_local_cache,
             transform=transform,
+            batch_size=batch_size,
             num_workers=num_workers,
             buffer_size=buffer_size,
+            return_index=return_index,
+            pad_tensors=pad_tensors,
         ),
         batch_size=batch_size,
         collate_fn=collate_fn,
@@ -96,6 +102,8 @@ def dataset_to_pytorch(
     transform: Optional[Union[Dict, Callable]] = None,
     tensors: Optional[Sequence[str]] = None,
     tobytes: Union[bool, Sequence[str]] = False,
+    return_index: bool = True,
+    pad_tensors: bool = True,
 ):
 
     import torch
@@ -108,13 +116,26 @@ def dataset_to_pytorch(
     if collate_fn is None:
         collate_fn = default_convert_fn if batch_size is None else default_collate_fn
 
+    if tensors is not None and "index" in tensors:
+        raise ValueError("index is not a tensor, to get index, pass return_index=True")
+
     tensors = map_tensor_keys(dataset, tensors)
     if isinstance(transform, dict):
-        tensors = list(transform.keys())
-        transform = PytorchTransformFunction(transform_dict=transform, tensors=tensors)
+        tensors = [k for k in transform.keys() if k != "index"]
+        transform = PytorchTransformFunction(transform_dict=transform)
     else:
         transform = PytorchTransformFunction(composite_transform=transform)
 
+    # check whether we have an empty tensor inside of tensors
+    for tensor_name in tensors:
+        tensor = dataset._get_tensor_from_root(tensor_name)
+        if len(tensor) == 0:
+            raise EmptyTensorError(
+                f" the dataset has an empty tensor {tensor_name}, pytorch dataloader can't be created."
+                f" Please either populate the tensor or pass tensors argument to .pytorch that excludes this"
+                f" tensor."
+            )
+
     if shuffle and num_workers > 0:
         return create_dataloader(
             dataset,
@@ -128,6 +149,8 @@ def dataset_to_pytorch(
             collate_fn,
             pin_memory,
             drop_last,
+            return_index,
+            pad_tensors,
         )
     else:
         return torch.utils.data.DataLoader(
@@ -140,6 +163,8 @@ def dataset_to_pytorch(
                 num_workers=num_workers,
                 shuffle=shuffle,
                 buffer_size=buffer_size,
+                return_index=return_index,
+                pad_tensors=pad_tensors,
             ),
             batch_size=batch_size,
             collate_fn=collate_fn,
