@@ -326,7 +326,7 @@ def hub_cloud_vstream_path(request, hub_cloud_dev_token):
         pytest.skip()
         return
 
-    path = f"{PYTEST_HUB_CLOUD_PROVIDER_BASE_ROOT}vstream_test"
+    path = f"{PYTEST_HUB_CLOUD_PROVIDER_BASE_ROOT}vstream_test_dataset"
 
     yield path
 
@@ -442,6 +442,18 @@ def video_paths():
     return paths
 
 
+@pytest.fixture
+def point_cloud_paths():
+    paths = {
+        "las": "point_cloud.las",
+    }
+
+    parent = get_dummy_data_path("point_cloud")
+    for k in paths:
+        paths[k] = os.path.join(parent, paths[k])
+    return paths
+
+
 @pytest.fixture
 def vstream_path(request):
     """Used with parametrize to use all video stream test datasets."""
